#!/bin/sh
#set -x
export PATH=$PATH:/bin:./
CUT="/bin/cut"
AWK="/bin/awk"
CAT="/bin/cat"
SED="/bin/sed"
GREP="/bin/grep"
. ./s3sfunc.sh

# usage
#./chklog.sh <check mode> <key> <key2> [<logfile>]
#check mode: TOOL1:1
#            TOOL2:2
#key       : OWN_KEY
#key2      : OTH_KEY
#logfile   : log path(Specify the log file you want to check.)

# メイン

TOOL_MODE=${1}

OWN_KEY=${2}
OTH_KEY=${3}
if [ "${4}" != "" ]
then
	LOG_FILENAME=${4}
else
	LOG_FILENAME="./log/scsi3PR_s3s_${OWN_KEY}.scsilog"
fi

OWN_KEY_LEN=${#OWN_KEY}
OTH_KEY_LEN=${#OTH_KEY}
if [ ${OWN_KEY_LEN} -lt 16 ]
then
	WK_KEY_LEN=$(( 16 - OWN_KEY_LEN ))
	PAD_KEY=`printf %0${WK_KEY_LEN}d 0`
	WK_KEY=`s3secho "${PAD_KEY}${OWN_KEY}"`
else
	WK_KEY=`s3secho "${OWN_KEY}"`
fi
WK_KEY=`echo ${WK_KEY} | awk '{ print(toupper($0)) }'`
OWNKY1=`s3secho ${WK_KEY} | ${CUT} -c 1-8`
OWNKY2=`s3secho ${WK_KEY} | ${CUT} -c 9-16`
if [ ${OTH_KEY_LEN} -lt 16 ]
then
	WK_KEY_LEN=$(( 16 - OTH_KEY_LEN ))
	PAD_KEY=`printf %0${WK_KEY_LEN}d 0`
	WK_KEY=`s3secho "${PAD_KEY}${OTH_KEY}"`
else
	WK_KEY=`s3secho "${OWN_KEY}"`
fi
WK_KEY=`echo ${WK_KEY} | awk '{ print(toupper($0)) }'`
OTHKY1=`s3secho ${WK_KEY} | ${CUT} -c 1-8`
OTHKY2=`s3secho ${WK_KEY} | ${CUT} -c 9-16`

s3secho "Start checking the log."

while read LINE
do
	HEAD=`s3secho "${LINE}" | "${GREP}" "start =====$"`
	if [ "${HEAD}" != "" ]
	then
		CONFI_NO=`s3secho "${LINE}" | "${AWK}" '{print $2}'`
		SEQ_ORD=0
		SEQ_RTN=0
		CMD_TYPE=""
		continue
	fi

	# "./"から始まる行は無視
	HEAD=`s3secho "${LINE}" | "${GREP}" "^\./.*"`
	if [ "${HEAD}" != "" ]
	then
		continue
	fi
	# "/dev"から始まる行は無視
	HEAD=`s3secho "${LINE}" | "${GREP}" "^/dev.*"`
	if [ "${HEAD}" != "" ]
	then
		continue
	fi
	# "DXFERP"が含まれる行は次の行以降に表示されるDUMP結果を解析
	HEAD=`s3secho "${LINE}" | "${AWK}" '{print $2}'`
	if [ "${HEAD}" = "DXFERP" ]
	then
		if [ "${CMD_TYPE}" = "READ RESERVATION" ]
		then
			DUMP_PARSING_FLAG=1
			# READ RESERVATIONができれば良いときは,
			# DUMP_PARSING_FLAGを0に戻す。
			if [ "${CONFI_NO}" = "16,14,20" -o "${CONFI_NO}" = "18,2" -o "${CONFI_NO}" = "24" ]
			then
				if [ "${SEQ_ORD}" = "1" ]
				then
					DUMP_PARSING_FLAG=0
				fi
			fi
		fi
		continue
	fi
	# "--ADDR--"から始まる行は無視
	HEAD=`s3secho "${LINE}" | "${GREP}" "^--ADDR--.*"`
	if [ "${HEAD}" != "" ]
	then
		continue
	fi
	# DUMP結果を解析
	HEAD=`s3secho "${LINE}" | "${AWK}" '{print $2}'`
	if [ "${HEAD}" = "|" ]
	then
		if [ "${DUMP_PARSING_FLAG}" = "1" ]
		then
			# 必要なのはリザーブキーが格納されている
			# DXFERPの8-16byte目だけで2行目以降のDUMP結果の解析は不要
			# なのでDUMP解析フラグはすぐ戻す。
			DUMP_PARSING_FLAG=0
			DUMPR1=`s3secho "${LINE}" | "${AWK}" '{print $5}'`
			DUMPR2=`s3secho "${LINE}" | "${AWK}" '{print $6}'`
		fi
		continue
	fi
	# 年月日で始まる行を解析
	HEAD=`s3secho "${LINE}" | "${GREP}" "^[0-9]\{4\}/[0-1][0-9]/[0-3][0-9] "`
	if [ "${HEAD}" != "" ]
	then
		# "start"で終わる場合は，SCSIコマンドの種別を取得
		HEAD=`s3secho "${LINE}" | "${GREP}" ".*start$"`
		if [ "${HEAD}" != "" ]
		then
			HEADD=`s3secho "${LINE}" | "${SED}" "s/.*] //g"`
			PRE_CMD_TYPE=${CMD_TYPE}
			PRE_SEQ_ORD=${SEQ_ORD}
			CMD_TYPE=`s3secho "${HEADD}" | "${SED}" "s/ start$//g"`
			if [ "${PRE_CMD_TYPE}" != "${CMD_TYPE}" ]
			then
				SEQ_ORD=$(( SEQ_ORD + 1 ))
			fi
			continue
		else
			# "start"で終わらない場合は，rcを取得する
			HEADD=`s3secho "${LINE}" | "${SED}" "s/.*rc=/rc=/g"`
			CMD_RC=`s3secho "${HEADD}" | "${AWK}" -F '[ =]' '{print $2}'`
			case "${CONFI_NO}" in
				"7,13" | "9,17" )
					case "${SEQ_ORD}" in
						"1" )
							# CLEAR
							if [ "${CMD_RC}" = "24" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"25" )
					case "${SEQ_ORD}" in
						"1" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								if [ "${DUMPR1}" = "00000000" -a "${DUMPR2}" = "00000000" ]
								then
									CMD_RTN=0
								else
									CMD_RTN=1
								fi
							else
								CMD_RTN=1
							fi
						;;
						"2" )
							# REGISTER
							if [ "${CMD_RC}" = "24" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"1,29" )
					case "${SEQ_ORD}" in
						"1" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								if [ "${DUMPR1}" = "00000000" -a "${DUMPR2}" = "00000000" ]
								then
									CMD_RTN=0
								else
									CMD_RTN=1
								fi
							else
								CMD_RTN=1
							fi
						;;
						"2" | "3" )
							# 2:REGISTER AND IGNORE EXISTING KEY
							# 3:RESERVE
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"2,30" | "26" )
					case "${SEQ_ORD}" in
						"1" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								if [ "${DUMPR1}" = "${OWNKY1}" -a "${DUMPR2}" = "${OWNKY2}" ]
								then
									CMD_RTN=0
								else
									CMD_RTN=1
								fi
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"3,31" | "28" )
					case "${SEQ_ORD}" in
						"1" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								if [ "${DUMPR1}" = "${OTHKY1}" -a "${DUMPR2}" = "${OTHKY2}" ]
								then
									CMD_RTN=0
								else
									CMD_RTN=1
								fi
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"27" )
					case "${SEQ_ORD}" in
						"1" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								if [ "${DUMPR1}" = "${OWNKY1}" -a "${DUMPR2}" = "${OWNKY2}" ]
								then
									CMD_RTN=0
								else
									CMD_RTN=1
								fi
							else
								CMD_RTN=1
							fi
						;;
						"2" )
							# REGISTER
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"8,15" | "ReserveClear" )
					case "${SEQ_ORD}" in
						"1" )
							# CLEAR
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"16,14,20" )
					case "${SEQ_ORD}" in
						"1" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"2" | "5" | "6" )
							# 2,5:REGISTER AND IGNORE EXISTING KEY
							# 6:RESERVE
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"3" )
							# PREEMPT AND ABORT
							if [ "${CMD_RC}" = "24" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"4" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								if [ "${DUMPR1}" = "00000000" -a "${DUMPR2}" = "00000000" ]
								then
									CMD_RTN=0
								else
									CMD_RTN=1
								fi
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"18,22" )
					case "${SEQ_ORD}" in
						"1" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"2" )
							# REGISTER AND IGNORE EXISTING KEY
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"3" )
							# PREEMPT AND ABORT
							if [ "${CMD_RC}" = "24" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"4" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								if [ "${DUMPR1}" = "${OWNKY1}" -a "${DUMPR2}" = "${OWNKY2}" ]
								then
									CMD_RTN=0
								else
									CMD_RTN=1
								fi
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
				"24" )
					case "${SEQ_ORD}" in
						"1" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"2" )
							# REGISTER AND IGNORE EXISTING KEY
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"3" )
							# PREEMPT AND ABORT
							if [ "${CMD_RC}" = "0" ]
							then
								CMD_RTN=0
							else
								CMD_RTN=1
							fi
						;;
						"4" )
							# READ RESERVATION
							if [ "${CMD_RC}" = "0" ]
							then
								if [ "${DUMPR1}" = "${OWNKY1}" -a "${DUMPR2}" = "${OWNKY2}" ]
								then
									CMD_RTN=0
								else
									CMD_RTN=1
								fi
							else
								CMD_RTN=1
							fi
						;;
						* )
							# ありえない
						;;
					esac
				;;
			esac
			# SEQ_RTNは0->1は許すが，1->0は許さない。
			# ただし,リトライの場合も更新する。
			if [ "${SEQ_RTN}" = "0" -o "${PRE_SEQ_ORD}" = "${SEQ_ORD}" ]
			then
				SEQ_RTN=${CMD_RTN}
			fi
		fi
		continue
	fi
	# "end ====="で結果を出力
	HEAD=`s3secho "${LINE}" | "${GREP}" "end =====$"`
	if [ "${HEAD}" != "" ]
	then
		ORDER_LINE=`"${CAT}" ./Scenario | "${GREP}" "${CONFI_NO}$"`
		ORDER=`s3secho "${ORDER_LINE}" | "${AWK}" -F / '{print $1}'`
		if [ "${SEQ_RTN}" = "0" ]
		then
			if [ "${TOOL_MODE}" = "1" ]
			then
				case "${CONFI_NO}" in
					"3,31" | "28" | "16,14,20" | "18,22" | "24" )
						SEQ_ORD=1
					;;
					*)
					;;
				esac
			else
				case "${CONFI_NO}" in
					"7,13" | "9,17" | "25" | "1,29" | "2,30" | "26" | "27" | "8,15" | "24" | "ReserveClear" )
						SEQ_ORD=1
					;;
					* )
					;;
				esac
			fi
			if [ "${SEQ_ORD}" = "0" ]
			then
				s3secho "The result of sequence ${ORDER}/14 is NG!!!(Could not judge.)"
			else
				s3secho "The result of sequence ${ORDER}/14 is OK."
			fi
		else
			s3secho "The result of sequence ${ORDER}/14 is NG!!!"
		fi
		continue
	fi

done < ${LOG_FILENAME}

s3secho "Finished checking the log."

exit 0
